ActorModel

Struct ActorModel 

pub struct ActorModel<A, C = (), H = ()>
where A: Actor, A::Msg: Ord, A::Timer: Ord, H: Clone + Debug + Hash,
{ pub actors: Vec<A>, pub cfg: C, pub init_history: H, pub init_network: Network<A::Msg>, pub lossy_network: LossyNetwork, pub max_crashes: usize, pub properties: Vec<Property<ActorModel<A, C, H>>>, pub record_msg_in: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>, pub record_msg_out: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>, pub within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> bool, }
Represents a system of Actors that communicate over a network. H indicates the type of history to maintain as auxiliary state, if any. See Auxiliary Variables in TLA for a thorough introduction to that concept. Use () if history is not needed to define the relevant properties of this system.

Fields

actors: Vec<A>cfg: Cinit_history: Hinit_network: Network<A::Msg>lossy_network: LossyNetworkmax_crashes: usize

Maximum number of actors that can be contemporarily crashed

properties: Vec<Property<ActorModel<A, C, H>>>record_msg_in: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>record_msg_out: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> bool

Implementations

Source

impl<A, C, H> ActorModel<A, C, H>
where A: Actor, A::Msg: Ord, A::Timer: Ord, H: Clone + Debug + Hash,

pub fn new(cfg: C, init_history: H) -> ActorModel<A, C, H>

Initializes an ActorModel with a specified configuration and history.

pub fn actor(self, actor: A) -> Self

Adds another Actor to this model.

pub fn actors(self, actors: impl IntoIterator<Item = A>) -> Self

Adds multiple Actors to this model.

pub fn init_network(self, init_network: Network<A::Msg>) -> Self

Defines the initial network.

pub fn lossy_network(self, lossy_network: LossyNetwork) -> Self

Defines whether the network loses messages or not.

pub fn max_crashes(self, max_crashes: usize) -> Self

Specifies the maximum number of actors that can be contemporarily crashed

pub fn property( self, expectation: Expectation, name: &'static str, condition: fn(&ActorModel<A, C, H>, &ActorModelState<A, H>) -> bool, ) -> Self

Adds a Property to this model.

pub fn record_msg_in( self, record_msg_in: fn(cfg: &C, history: &H, _: Envelope<&A::Msg>) -> Option<H>, ) -> Self

Defines whether/how an incoming message contributes to relevant history. Returning Some(new_history) updates the relevant history, while None does not.

pub fn record_msg_out( self, record_msg_out: fn(cfg: &C, history: &H, _: Envelope<&A::Msg>) -> Option<H>, ) -> Self

Defines whether/how an outgoing message contributes to relevant history. Returning Some(new_history) updates the relevant history, while None does not.

pub fn within_boundary( self, within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> bool, ) -> Self

Indicates whether a state is within the state space that should be model checked.