pub struct ActorModel<A, C = (), H = ()>{
pub actors: Vec<A>,
pub cfg: C,
pub init_history: H,
pub init_network: Network<A::Msg>,
pub lossy_network: LossyNetwork,
pub max_crashes: usize,
pub properties: Vec<Property<ActorModel<A, C, H>>>,
pub record_msg_in: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>,
pub record_msg_out: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>,
pub within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> bool,
}Expand description
Represents a system of Actors that communicate over a network. H indicates the type of
history to maintain as auxiliary state, if any. See Auxiliary Variables in
TLA for a thorough
introduction to that concept. Use () if history is not needed to define the relevant
properties of this system.
Fields
actors: Vec<A>cfg: Cinit_history: Hinit_network: Network<A::Msg>lossy_network: LossyNetworkmax_crashes: usizeMaximum number of actors that can be contemporarily crashed
properties: Vec<Property<ActorModel<A, C, H>>>record_msg_in: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>record_msg_out: fn(cfg: &C, history: &H, envelope: Envelope<&A::Msg>) -> Option<H>within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> boolImplementations
Sourceimpl<A, C, H> ActorModel<A, C, H>
impl<A, C, H> ActorModel<A, C, H>
Sourcepub fn new(cfg: C, init_history: H) -> ActorModel<A, C, H>
pub fn new(cfg: C, init_history: H) -> ActorModel<A, C, H>
Initializes an ActorModel with a specified configuration and history.
Sourcepub fn actors(self, actors: impl IntoIterator<Item = A>) -> Self
pub fn actors(self, actors: impl IntoIterator<Item = A>) -> Self
Adds multiple Actors to this model.
Sourcepub fn init_network(self, init_network: Network<A::Msg>) -> Self
pub fn init_network(self, init_network: Network<A::Msg>) -> Self
Defines the initial network.
Sourcepub fn lossy_network(self, lossy_network: LossyNetwork) -> Self
pub fn lossy_network(self, lossy_network: LossyNetwork) -> Self
Defines whether the network loses messages or not.
Sourcepub fn max_crashes(self, max_crashes: usize) -> Self
pub fn max_crashes(self, max_crashes: usize) -> Self
Specifies the maximum number of actors that can be contemporarily crashed
Sourcepub fn property(
self,
expectation: Expectation,
name: &'static str,
condition: fn(&ActorModel<A, C, H>, &ActorModelState<A, H>) -> bool,
) -> Self
pub fn property( self, expectation: Expectation, name: &'static str, condition: fn(&ActorModel<A, C, H>, &ActorModelState<A, H>) -> bool, ) -> Self
Adds a Property to this model.
Sourcepub fn record_msg_in(
self,
record_msg_in: fn(cfg: &C, history: &H, _: Envelope<&A::Msg>) -> Option<H>,
) -> Self
pub fn record_msg_in( self, record_msg_in: fn(cfg: &C, history: &H, _: Envelope<&A::Msg>) -> Option<H>, ) -> Self
Defines whether/how an incoming message contributes to relevant history. Returning
Some(new_history) updates the relevant history, while None does not.
Sourcepub fn record_msg_out(
self,
record_msg_out: fn(cfg: &C, history: &H, _: Envelope<&A::Msg>) -> Option<H>,
) -> Self
pub fn record_msg_out( self, record_msg_out: fn(cfg: &C, history: &H, _: Envelope<&A::Msg>) -> Option<H>, ) -> Self
Defines whether/how an outgoing message contributes to relevant history. Returning
Some(new_history) updates the relevant history, while None does not.
Sourcepub fn within_boundary(
self,
within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> bool,
) -> Self
pub fn within_boundary( self, within_boundary: fn(cfg: &C, state: &ActorModelState<A, H>) -> bool, ) -> Self
Indicates whether a state is within the state space that should be model checked.